@@ -3,6 +3,7 @@ import torch
 from torch4keras.snippets import metric_mapping, ProgbarLogger, Callback, CallbackList, BaseLogger, History, TqdmProgressBar
 from collections import OrderedDict
 from inspect import isfunction
+import os
 
 
 class Trainer:
@@ -12,6 +13,9 @@ class Trainer:
     """
     def __init__(self, module=None):
         super(Trainer, self).__init__()
+        self.initialize(module)
+    
+    def initialize(self, module=None):
         # 这里主要是为了外面调用用到
         self.global_step, self.local_step, self.total_steps, self.epoch, self.steps_per_epoch, self.train_dataloader = 0, 0, 0, 0, None, None
         self.resume_step, self.resume_epoch = 0, 0
@@ -19,40 +23,22 @@ class Trainer:
         self.callbacks = []
         # 传入Module实例方式
         if module is not None:
+            assert isinstance(module, nn.Module), 'Args `module` only support nn.Module format'
             self.module = module
-    
-    def save_steps_params(self, save_path):
-        '''保存训练过程参数
+        # 是否运行Callbacks，目前主要是在DDP模式下运用
+        self.run_callbacks = True
+        # loss是否立即backward
+        self.loss_backward = True
 
-        :param save_path: str, 训练过程参数保存路径
-        '''
-        step_params = {'resume_step': (self.local_step+1) % self.steps_per_epoch, 
-                       'resume_epoch': self.epoch + (self.local_step+1) // self.steps_per_epoch}
-        torch.save(step_params, save_path)
-
-    def load_steps_params(self, save_path):
-        '''导入训练过程参数
-        
-        :param save_path: str, 训练过程参数保存路径
-        '''
-        step_params = torch.load(save_path)
-        self.resume_step = step_params['resume_step'] 
-        self.resume_epoch = step_params['resume_epoch']
-        return step_params
-
-    def forward(self, *inputs, **kwargs):
-        # 如果传入了网络结构module，则调用module的forward
-        # 如果是继承方式，则调用自身的forward
-        return self.get_module().forward(*inputs, **kwargs)
-
-    def compile(self, loss, optimizer, scheduler=None, clip_grad_norm=None, use_amp=False, metrics=None, stateful_metrics=None, grad_accumulation_steps=1, **kwargs):
+    def compile(self, loss, optimizer, scheduler=None, clip_grad_norm=None, mixed_precision=False, metrics=None, 
+                stateful_metrics=None, grad_accumulation_steps=1, **kwargs):
         '''complile: 定义loss, optimizer, metrics等参数
         
         :param loss: loss
         :param optimizer: 优化器
         :param scheduler: scheduler
         :param clip_grad_norm: bool, 是否使用梯度裁剪, 默认为False
-        :param use_amp: bool, 是否使用混合精度，默认为False
+        :param mixed_precision: bool, 是否使用混合精度，默认为False
         :param metrics: str/List[str]/dict, 训练过程中需要打印的指标, loss相关指标默认会打印, 目前支持accuracy, 也支持自定义metric，形式为{key: func}
         :param stateful_metrics: List[str], 不滑动平均仅进行状态记录的metric，指标抖动会更加明显
         :param grad_accumulation_steps: int, 梯度累积步数，默认为1
@@ -63,10 +49,10 @@ class Trainer:
         self.optimizer = optimizer
         self.scheduler = scheduler
         self.clip_grad_norm = clip_grad_norm
-        self.use_amp = use_amp
-        if use_amp:
-            from torch.cuda.amp import autocast
-            self.autocast = autocast
+        assert mixed_precision in {True, False, 'fp16', 'bf16'}
+        self.mixed_precision = 'fp16' if mixed_precision is True else mixed_precision
+        if self.mixed_precision:
+            self.autocast = torch.cuda.amp.autocast
             self.scaler = torch.cuda.amp.GradScaler()
 
         # 训练过程观测的指标
@@ -95,15 +81,13 @@ class Trainer:
 
         # 进度条参数
         self.tqdmbar = kwargs.get('tqdmbar', False)
-
-    def args_segmentate(self, train_X):
-        '''参数是否展开
-        '''
+    
+    def _forward(self, train_X):
+        # 如果传入了网络结构module，则调用module的forward
+        # 如果是继承方式，则调用自身的forward
         if isinstance(train_X, torch.Tensor):  # tensor不展开
-            pass
-        elif self.get_module().forward.__code__.co_argcount >= 3:
-            return True
-        return False
+            return self.get_module().forward(train_X)
+        return self.get_module().forward(*train_X)
 
     def train_step(self, train_X, train_y):
         '''forward并返回loss
@@ -112,14 +96,16 @@ class Trainer:
         :param train_y: torch.Tensor/List[torch.Tensor], 标签信息
         :return: [output, loss, loss_detail] output: torch.Tensor/List[torch.Tensor], 模型输出; loss: nn.Tensor, 计算得到的loss; loss_detail: dict[nn.Tensor], 具体的各个loss
         '''
-        if self.use_amp:
-            with self.autocast():
-                output = self.forward(*train_X) if self.args_segmentate(train_X) else self.forward(train_X)
+        # 计算loss
+        if self.mixed_precision:
+            with self.autocast(dtype=torch.float16 if self.mixed_precision=='fp16' else torch.bfloat16):
+                output = self._forward(train_X)
                 loss_detail = self.criterion(output, train_y)
         else:
-            output = self.forward(*train_X) if self.args_segmentate(train_X) else self.forward(train_X)
+            output = self._forward(train_X)
             loss_detail = self.criterion(output, train_y)
 
+        # 整理loss
         if isinstance(loss_detail, torch.Tensor):
             loss = loss_detail
             loss_detail = {}
@@ -135,13 +121,15 @@ class Trainer:
         # 梯度累积
         loss = loss / self.grad_accumulation_steps if self.grad_accumulation_steps > 1 else loss
 
-        # backward
-        self.scale_before_step = 0
-        if self.use_amp:  # 混合精度
-            self.scale_before_step = self.scaler.get_scale()
-            self.scaler.scale(loss).backward(retain_graph=self.retain_graph)
-        else:
-            loss.backward(retain_graph=self.retain_graph)
+        # loss.backward
+        if self.loss_backward:
+            self.scale_before_step = 0
+            if self.mixed_precision:  # 混合精度
+                self.scale_before_step = self.scaler.get_scale()
+                self.scaler.scale(loss).backward(retain_graph=self.retain_graph)
+            else:
+                loss.backward(retain_graph=self.retain_graph)
+                
         return output, loss, loss_detail
 
     def fit(self, train_dataloader, steps_per_epoch=None, epochs=1, callbacks=None, verbose=1):
@@ -161,7 +149,8 @@ class Trainer:
         self.total_steps = self.steps_per_epoch * epochs
         self.train_dataloader = train_dataloader  # 设置为成员变量，可由外部的callbacks进行修改
         train_dataloader_iter = iter(self.train_dataloader)  # 循环epoch时不重生成
-        
+        verbose = self.verbose if hasattr(self, 'verbose') else verbose
+
         # callbacks设置
         if callbacks is None:
             callbacks = []
@@ -169,16 +158,19 @@ class Trainer:
             callbacks = [callbacks]
         for callback in callbacks:
             assert isinstance(callback, Callback), "Args `callbacks` only support Callback() inputs"
-        
-        # 进度条
-        if self.tqdmbar:
-            progbarlogger = TqdmProgressBar(stateful_metrics=self.stateful_metrics)
-        else:
-            progbarlogger = ProgbarLogger(stateful_metrics=self.stateful_metrics)
-            
+
         history = History()
-        master_rank = self.master_rank if hasattr(self, 'master_rank') else None
-        self.callbacks = CallbackList([BaseLogger(self.stateful_metrics), progbarlogger] + callbacks + [history], master_rank=master_rank)
+        callbacks_ = [BaseLogger(self.stateful_metrics)]
+
+        # 进度条
+        if verbose:
+            if self.tqdmbar:
+                progbarlogger = TqdmProgressBar(stateful_metrics=self.stateful_metrics)
+            else:
+                progbarlogger = ProgbarLogger(stateful_metrics=self.stateful_metrics)
+            callbacks_.append(progbarlogger)
+        callbacks_  += callbacks + [history]
+        self.callbacks = CallbackList(callbacks_, run_callbacks=self.run_callbacks)
         callback_trainer = self
         callback_model = self.get_module()
         self.callbacks.set_trainer(callback_trainer)
@@ -204,7 +196,8 @@ class Trainer:
             # resume_step：判断local_step的起点，以及进度条的起始位置
             resume_step = self.resume_step if epoch==self.resume_epoch else 0
             self.callbacks.on_epoch_begin(self.global_step, self.epoch)
-            progbarlogger.seen = resume_step  # 这里设置进度条的seen，在callbacks中也会修改
+            if verbose:
+                progbarlogger.seen = resume_step  # 这里设置进度条的seen，在callbacks中也会修改
             
             for local_step in range(resume_step, self.steps_per_epoch):
                 self.local_step = local_step
@@ -241,7 +234,7 @@ class Trainer:
                 if (self.global_step+1) % self.grad_accumulation_steps == 0:
                     skip_scheduler = False
                     # 混合精度
-                    if self.use_amp:
+                    if self.mixed_precision:
                         self.scaler.unscale_(self.optimizer)
                         if self.clip_grad_norm is not None:  # 梯度裁剪
                             torch.nn.utils.clip_grad_norm_(self.parameters(), self.clip_grad_norm)
@@ -265,7 +258,7 @@ class Trainer:
                 logs.update({'loss': self.loss.item()})
                 logs_loss_detail = {k: v.item() if isinstance(v, torch.Tensor) else v for k, v in self.loss_detail.items()}
                 logs.update(logs_loss_detail)
-                if self.global_step == resume_step:
+                if verbose and (self.global_step == resume_step):
                     progbarlogger.add_metrics(list(logs_loss_detail.keys()), add_position=1)
                     
                 # 添加metrics至log打印
@@ -273,7 +266,7 @@ class Trainer:
                     perf = metric_mapping(metric, func, self.output, self.train_y)  # 内置的一些accuracy指标
                     if perf is not None:
                         if isfunction(metric):  # 直接传入回调函数(无key)
-                            if self.global_step == resume_step:
+                            if verbose and (self.global_step == resume_step):
                                 progbarlogger.add_metrics(list(perf.keys()))
                             logs.update(perf)
                         elif isinstance(metric, str):  # 直接传入回调函数(有key)
@@ -294,11 +287,11 @@ class Trainer:
         '''模型预测，调用forward()
 
         :param train_X: torch.Tensor, 预测用的数据集
-        :param return_all: None/int, 若返回为多个时候指定仅返回第几个，默认为None表示否返回
+        :param return_all: None/int, 若返回为多个时候指定仅返回第几个，默认为None表示全部返回
         :return: Any, 预测输出
         '''
         self.get_module().eval()
-        output = self.forward(*train_X) if self.args_segmentate(train_X) else self.forward(train_X)
+        output = self._forward(train_X)
         if return_all is None:
             return output
         elif isinstance(output, (tuple, list)) and isinstance(return_all, int) and return_all < len(output):
@@ -306,6 +299,25 @@ class Trainer:
         else:
             raise ValueError('Return format error')
 
+    def load_steps_params(self, save_path):
+        '''导入训练过程参数
+        
+        :param save_path: str, 训练过程参数保存路径
+        '''
+        step_params = torch.load(save_path)
+        self.resume_step = step_params['resume_step'] 
+        self.resume_epoch = step_params['resume_epoch']
+        return step_params
+
+    def save_steps_params(self, save_path):
+        '''保存训练过程参数
+
+        :param save_path: str, 训练过程参数保存路径
+        '''
+        step_params = {'resume_step': (self.local_step+1) % self.steps_per_epoch, 
+                       'resume_epoch': self.epoch + (self.local_step+1) // self.steps_per_epoch}
+        torch.save(step_params, save_path)
+
     def load_weights(self, load_path, strict=True, mapping={}):
         '''加载模型权重
 
@@ -332,9 +344,47 @@ class Trainer:
             state_dict_raw[k] = v
         torch.save(state_dict_raw, save_path)
 
+    def resume_from_checkpoint(self, model_path=None, optimizer_path=None, step_params_path=None):
+        '''同时加载模型、优化器、训练过程参数
+
+        :param model_path: str, 模型文件路径
+        :param optimizer_path: str, 优化器文件路径
+        :param step_params_path: str, 训练过程参数保存路径
+        '''
+        # 加载模型权重
+        if model_path:
+            self.load_weights(model_path)
+        # 加载优化器，断点续训使用
+        if optimizer_path:
+            state_dict = torch.load(optimizer_path, map_location='cpu')
+            self.optimizer.load_state_dict(state_dict)
+        # 加载训练进度参数，断点续训使用
+        self.load_steps_params(step_params_path)
+
+    def save_to_checkpoint(self, model_path=None, optimizer_path=None, step_params_path=None):
+        '''同时保存模型、优化器、训练过程参数
+
+        :param model_path: str, 模型文件路径
+        :param optimizer_path: str, 优化器文件路径
+        :param step_params_path: str, 训练过程参数保存路径
+        '''
+        if model_path:
+            save_dir = os.path.dirname(model_path)
+            os.makedirs(save_dir, exist_ok=True)
+            self.save_weights(model_path)
+        if optimizer_path:
+            save_dir = os.path.dirname(optimizer_path)
+            os.makedirs(save_dir, exist_ok=True)
+            torch.save(self.optimizer.state_dict(), optimizer_path)
+        if step_params_path:
+            save_dir = os.path.dirname(step_params_path)
+            os.makedirs(save_dir, exist_ok=True)
+            self.save_steps_params(step_params_path)
+
     def get_module(self):
         '''返回nn.Module模块
         '''
+        if isinstance(self, nn.Module): return self
         return self.module if hasattr(self, 'module') else self
 
 
@@ -352,16 +402,44 @@ class BaseModelDP(nn.DataParallel, BaseModel):
     def __init__(self, *args, **kwargs):
         BaseModel.__init__(self)
         nn.DataParallel.__init__(self, *args, **kwargs)
-
+    
+    # # 使用dp自身的forward()而不是module.forward()
+    # def _forward(self, train_X):
+    #     if isinstance(train_X, torch.Tensor):
+    #         return self.forward(train_X)
+    #     return self.forward(*train_X)
 
 class BaseModelDDP(nn.parallel.DistributedDataParallel, BaseModel):
     '''DistributedDataParallel模式使用多gpu的方法, 父类顺序颠倒也会出问题
     '''
     def __init__(self, *args, master_rank=0, **kwargs):
-        self.master_rank = master_rank  # 用于记录打印条的rank
         BaseModel.__init__(self)
         nn.parallel.DistributedDataParallel.__init__(self, *args, **kwargs)
 
+        # 默认仅对master_rank=0打印信息
+        assert isinstance(master_rank, (int, list, tuple)), 'Args `master_rank` only supoorts int, list, tuple'
+        if isinstance(master_rank, int):
+            master_rank = [master_rank]
+        self.verbose = (torch.distributed.get_rank() in master_rank)
+
 
 TrainerDP = BaseModelDP
-TrainerDDP = BaseModelDDP
\ No newline at end of file
+TrainerDDP = BaseModelDDP
+
+
+def add_trainer(obj):
+    '''为对象添加Triner对应的方法
+    '''
+    if isinstance(obj, (Trainer, TrainerDP, TrainerDDP, BaseModel, BaseModelDP, BaseModelDDP)):
+        return obj
+    
+    if isinstance(obj, nn.Module):
+        import types
+        for k in dir(Trainer):
+            if k.startswith('__') and k.endswith('__'):
+                continue
+            elif (k == 'forward') and hasattr(obj, 'forward'):
+                continue
+            exec(f'obj.{k} = types.MethodType(Trainer.{k}, obj)')
+        obj.initialize()
+    return obj
\ No newline at end of file
